@@ -2,25 +2,16 @@
 #'
 #' returns world coastline at specified scale
 #'
-#' @param scale scale of map to return, one of \code{110}, \code{50}, \code{10}
-#' or \code{'small'}, \code{'medium'}, \code{'large'}
-#'
-#' @param returnclass 'sp' default or 'sf' for Simple Features
+#' @inherit ne_download
 #'
 #' @examples
 #' if (requireNamespace("rnaturalearthdata")) {
-#'   sldf_coast <- ne_coastline()
-#'
-#'   if (require(sp)) {
-#'     plot(sldf_coast)
-#'   }
+#'   coast <- ne_coastline()
+#'   plot(coast)
 #' }
 #'
-#' @return \code{SpatialLinesDataFrame} or \code{sf}
-#'
 #' @export
-ne_coastline <- function(scale = 110,
-                         returnclass = c("sp", "sf")) {
+ne_coastline <- function(scale = 110L, returnclass = c("sf", "sv")) {
   returnclass <- match.arg(returnclass)
 
   if (returnclass == "sp") {
@@ -28,7 +19,7 @@ ne_coastline <- function(scale = 110,
   }
 
   # check for the data packages and try to install if not there
-  if (scale == 10) {
+  if (scale == 10L) {
     check_rnaturalearthhires()
   } else {
     check_rnaturalearthdata()
@@ -40,14 +31,14 @@ ne_coastline <- function(scale = 110,
   # choose which map based on scale
   sldf <- NULL
 
-  if (scale == 110) {
+  if (scale == 110L) {
     sldf <- rnaturalearthdata::coastline110
-  } else if (scale == 50) {
+  } else if (scale == 50L) {
     sldf <- rnaturalearthdata::coastline50
-  } else if (scale == 10) {
+  } else if (scale == 10L) {
     sldf <- rnaturalearthhires::coastline10
   }
 
-  # convert to sf if chosen
-  ne_as_sp(sldf, returnclass)
+  # Convert to the desired class
+  convert_spatial_class(sldf, returnclass)
 }
